package bsplines;

/*
Tutorial on LevelSet and BSpline deformable model techniques 
for image segmentation
Copyright (C) Diane Lingrand (Diane.Lingrand@unice.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.BorderFactory; 
import javax.swing.JSpinner.NumberEditor;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.io.*;
import javax.swing.filechooser.FileFilter;
import java.util.jar.*;
import java.net.URL;

/**
 * main class
 * all interface widgets (buttons, labels, panels, ..) are here 
 *
 * @author <a href="mailto:Diane.Lingrand@unice.fr">DianeLingrand</a>
**/

public class BSFrame extends JPanel implements ActionListener, ChangeListener {
    private static final double LAMBDA1 = 0.001;
    private static final double LAMBDA2 = 0.001;
    private static final double LAMBDA3 = 1.0;
    private static final Color FUSCHIA = new Color(200,0,200);

    private boolean isAnApplet = true;

    private boolean modelSplines = true; // true if BSplines, false if LevelSet methods
    private SegPanel panel;

    private JButton clearButton, distMapShow;
    private JButton criterionButton;
    // PUIS METTRE DES CRITERES D'ARRET
    private JRadioButton meanCritButton, varCritButton1, varCritButton2, gradCritButton1, gradCritButton2, gradCritButton3, gradCritButton4;

    private JButton modelChoice;
    private JButton loadImage, saveImage, loadBSplines, saveBSplines, loadParameters, saveParameters;
    private JSpinner jspin1, jspin2, jspin3;
    private JButton cutSplines, checkOverlap, applyCrit, regulPoints, reorientCurv, loopEvol;
    private JButton helpButton;
    private JButton addPoints, delPoints;
    private JButton captureMode, smoothingMode;
    private JSpinner setMinDist, setMaxDist;

    private JLabel pmc, pmc2, pmc3, pfl, prl, pll, pel, ppl, minDistLab, maxDistLab;

    private boolean inLoop = false;
    private Thread loopThread;
    private LoopRun lr;

    public BSFrame(String filename) {
	super();
	setLayout(new BorderLayout());

	if(filename == null) {
	    if(modelSplines)
		panel = new BSPanel(LAMBDA1, LAMBDA2, LAMBDA3);
	    else
		panel = new LSPanel(LAMBDA1, LAMBDA2, LAMBDA3);
	}
	else {
	    if(modelSplines)
		panel = new BSPanel(filename, LAMBDA1, LAMBDA2, LAMBDA3);
	    else
		panel = new LSPanel(filename, LAMBDA1, LAMBDA2, LAMBDA3);
	}

	lr = new LoopRun(panel);
	loopThread = new Thread(lr);

	JPanel buttonPanel = new JPanel();
	add(buttonPanel,BorderLayout.NORTH);

	add(panel,BorderLayout.CENTER);

	JPanel rightButtonPanel = new JPanel();
	add(rightButtonPanel,BorderLayout.EAST);

	// choosing method between BSplines or LevelSet
	JPanel panelModelChoice = new JPanel(new GridLayout(0, 1));
	panelModelChoice.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	buttonPanel.add(panelModelChoice);
	pmc = new JLabel("Model :");
	pmc.setForeground(FUSCHIA);
	panelModelChoice.add(pmc);
	if(modelSplines)
	    modelChoice = new JButton("change to LevelSet Model"); 
	else
	    modelChoice = new JButton("change to BSplines Model");
	panelModelChoice.add(modelChoice);
	modelChoice.addActionListener(this);
	
	//capture on/off
	pmc2 = new JLabel("Capture :");
	pmc2.setForeground(FUSCHIA);
	panelModelChoice.add(pmc2);
	if(panel.getCapture())
	    captureMode = new JButton("disable capture");
	else
	    captureMode = new JButton("enable capture");
	captureMode.addActionListener(this);
	panelModelChoice.add(captureMode);

	//gaussian filter on the image
	pmc3 = new JLabel("Smoothing the image");
	pmc3.setForeground(FUSCHIA);
	panelModelChoice.add(pmc3);
	if(panel.getSmoothing())
	    smoothingMode = new JButton("disable smoothing");
	else
	    smoothingMode = new JButton("enable smoothing");
	smoothingMode.addActionListener(this);
	panelModelChoice.add(smoothingMode);

	// how to load / save an image of a spline
	JPanel panelLoadSave = new JPanel(new GridLayout(0, 1));
	panelLoadSave.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	buttonPanel.add(panelLoadSave);
	pfl = new JLabel("Files :");
	pfl.setForeground(FUSCHIA);
	panelLoadSave.add(pfl);
	loadImage = new JButton("Load an image"); panelLoadSave.add(loadImage);
	loadImage.addActionListener(this);
	saveImage = new JButton("Save an image"); panelLoadSave.add(saveImage);
	saveImage.addActionListener(this);
	//	saveImage.setEnabled(false);
	loadBSplines = new JButton("Load BSplines"); panelLoadSave.add(loadBSplines);
	loadBSplines.addActionListener(this);
	saveBSplines = new JButton("Save the BSplines"); panelLoadSave.add(saveBSplines);
	saveBSplines.addActionListener(this);
	loadParameters = new JButton("Load parameters"); panelLoadSave.add(loadParameters);
	loadParameters.addActionListener(this);
	saveParameters = new JButton("Save parameters"); panelLoadSave.add(saveParameters);
	saveParameters.addActionListener(this);

	// how to clear all the BSplines

	ImageIcon clearButtonIcon = null;
	URL imageURL = BSApplet.class.getResource("images/clearIcon.gif");
	if(imageURL != null)
	    clearButtonIcon = new ImageIcon(imageURL);

	if(clearButtonIcon != null)
	    clearButton = new JButton(clearButtonIcon);
	else
	    clearButton = new JButton("Clear BSPlines");
	clearButton.setToolTipText("clear all the BSplines");
	clearButton.setVisible(true);
	clearButton.addActionListener( new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			panel.clearBSplines();
		}
	    });
	buttonPanel.add(clearButton);

	// how to show the distance map. Values must be beetween +-Max(w,h)
	distMapShow = new JButton("show the distances map");
	distMapShow.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    panel.distMapShow();
		}
	    });
	buttonPanel.add(distMapShow);
	if(modelSplines)
	    distMapShow.setEnabled(false);

	// how to choose or change the evolution criterion 
	JPanel panelRadio = new JPanel(new GridLayout(0, 1));
	panelRadio.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	prl = new JLabel("Evolution criterion :");
	prl.setForeground(FUSCHIA);
	panelRadio.add(prl);

	meanCritButton = new JRadioButton("Mean");
	meanCritButton.setSelected(true);
	meanCritButton.addActionListener(this);
	varCritButton1 = new JRadioButton("Variance 1");
	varCritButton1.addActionListener(this);
	varCritButton2 = new JRadioButton("Variance 2");
	varCritButton2.addActionListener(this);
	gradCritButton1 = new JRadioButton("Gradient 1");
	gradCritButton1.addActionListener(this);
	gradCritButton2 = new JRadioButton("Gradient 2");
	gradCritButton2.addActionListener(this);
	gradCritButton3 = new JRadioButton("Gradient 3");
	gradCritButton3.addActionListener(this);
	gradCritButton4 = new JRadioButton("Gradient 4");
	gradCritButton4.addActionListener(this);
	panelRadio.add(meanCritButton);
	panelRadio.add(varCritButton1);
	panelRadio.add(varCritButton2);
	panelRadio.add(gradCritButton1);
	panelRadio.add(gradCritButton2);
	panelRadio.add(gradCritButton3);
	panelRadio.add(gradCritButton4);

	ButtonGroup group = new ButtonGroup();
	group.add(meanCritButton);
	group.add(varCritButton1);
	group.add(varCritButton2);
	group.add(gradCritButton1);
	group.add(gradCritButton2);
	group.add(gradCritButton3);
	group.add(gradCritButton4);
	buttonPanel.add(panelRadio);

	// how to modify the weights in the criterion
	JPanel panelLambda = new JPanel(new GridLayout(0, 1));
	panelLambda.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	pll = new JLabel("Criterion weights :");
	pll.setForeground(FUSCHIA);
	panelLambda.add(pll);
	JLabel l1l = new JLabel("  \u03BB 1 :"); // lambda 1 in Unicode
	l1l.setToolTipText("lambda 1");
	panelLambda.add(l1l);
	jspin1 = new JSpinner(new SpinnerNumberModel(LAMBDA1,0.0,100.0, 0.0001));
	jspin1.setEditor(new NumberEditor(jspin1,"##.#####"));
	panelLambda.add(jspin1);
	jspin1.addChangeListener(this);
	JLabel l2l = new JLabel("  \u03BB 2 :"); // lambda 2 in Unicode
	l2l.setToolTipText("lambda 2");
	panelLambda.add(l2l);
	jspin2 = new JSpinner(new SpinnerNumberModel(LAMBDA1,0.0,100.0, 0.0001));
	jspin2.setEditor(new NumberEditor(jspin2,"##.#####"));
	panelLambda.add(jspin2);
	jspin2.addChangeListener(this);
	JLabel l3l = new JLabel("  \u03BB 3 :"); // lambda 3 in Unicode
	l3l.setToolTipText("lambda 3");
	panelLambda.add(l3l);
	jspin3 = new JSpinner(new SpinnerNumberModel(LAMBDA3,0,100, 1));
	jspin3.setEditor(new NumberEditor(jspin3,"##.##"));
	jspin3.addChangeListener(this);
	panelLambda.add(jspin3);
	buttonPanel.add(panelLambda);


	// how to control evolution
	JPanel panelEvolution = new JPanel(new GridLayout(0, 1));
	pel = new JLabel("Control of the algorithm steps :");
	pel.setForeground(FUSCHIA);
	panelEvolution.add(pel);
	cutSplines = new JButton("Cut Splines");
	checkOverlap = new JButton("Check overlapping");
	applyCrit = new JButton("Apply forces");
	regulPoints = new JButton("Regul the points");
	reorientCurv = new JButton("Re-orient the curve");
	loopEvol = new JButton("Run the loop");
	loopEvol.setBackground(Color.WHITE);
	loopEvol.setForeground(Color.RED);

	cutSplines.addActionListener(this);
	checkOverlap.addActionListener(this);
	applyCrit.addActionListener(this);
	regulPoints.addActionListener(this);
	reorientCurv.addActionListener(this);
	loopEvol.addActionListener(this);

	panelEvolution.add(cutSplines);
	panelEvolution.add(checkOverlap);
	panelEvolution.add(applyCrit);
	panelEvolution.add(regulPoints);
	panelEvolution.add(reorientCurv);
	panelEvolution.add(loopEvol);

	panelEvolution.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	buttonPanel.add(panelEvolution);

	// Help menu
	helpButton = new JButton("Help");
	helpButton.setForeground(Color.WHITE);
	helpButton.setBackground(FUSCHIA);
	helpButton.addActionListener(this);
	buttonPanel.add(helpButton);
	
	// how to add/delete points
	JPanel panelPoints= new JPanel(new GridLayout(0, 1));
	panelPoints.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	rightButtonPanel.add(panelPoints);

	ppl = new JLabel("Managing points and their relative distances :");
	ppl.setForeground(FUSCHIA);
	panelPoints.add(ppl);

	addPoints = new JButton("Add points to BSplines");
	panelPoints.add(addPoints);
	addPoints.addActionListener(this);
	delPoints = new JButton("Delete points of BSplines");
	panelPoints.add(delPoints);
	delPoints.addActionListener(this);

	minDistLab = new JLabel("Minimal distance between two P points");
	panelPoints.add(minDistLab);
	setMinDist = new JSpinner(new SpinnerNumberModel(BSpline.CLOSE,2,1000,1));
	setMinDist.setEditor(new NumberEditor(setMinDist,"####"));
	setMinDist.addChangeListener(this);
	panelPoints.add(setMinDist);

	maxDistLab = new JLabel("Maximal distance between two P points");
	panelPoints.add(maxDistLab);
	setMaxDist = new JSpinner(new SpinnerNumberModel(BSpline.DISTANT,10,1000,1));
	setMaxDist.setEditor(new NumberEditor(setMaxDist,"####"));
	setMaxDist.addChangeListener(this);
	panelPoints.add(setMaxDist);

    }

    public BSFrame(boolean b) {
	this(null);
	isAnApplet = b;
    }
    public BSFrame(boolean b, String filename) {
	this(filename);
	isAnApplet = b;
    }

    public void evolve(boolean b) {
	panel.evolve(b);
    }

    /** Listens to the spinners **/
    public void stateChanged(ChangeEvent e) {
	Object o = e.getSource();
	if(o == jspin1)
	    panel.setLambda1(((SpinnerNumberModel) jspin1.getModel()).getNumber().doubleValue());
	else if(o == jspin2)
	    panel.setLambda2(((SpinnerNumberModel) jspin2.getModel()).getNumber().doubleValue());
	else if(o == jspin3)
	    panel.setLambda3(((SpinnerNumberModel) jspin3.getModel()).getNumber().doubleValue());
	else if(o == setMinDist) {
	    panel.setClose(((SpinnerNumberModel) setMinDist.getModel()).getNumber().intValue());
	    panel.regulDistances(0);
	    panel.repaint();
	}
	else if(o == setMaxDist) {
	    panel.setDistant(((SpinnerNumberModel) setMaxDist.getModel()).getNumber().intValue());
	    panel.regulDistances(0);
	    panel.repaint();
	}
	
    }


   /** Listens to the radio buttons. */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
	Object o = e.getSource();
	
	if(o == modelChoice) {
	    try{ loopThread.join(); }  catch(InterruptedException ie) { }

	    if(modelSplines) {
		System.out.println("changing to levelSet");
		cutSplines.setEnabled(false);
		checkOverlap.setEnabled(false);
		regulPoints.setText("Regul distance map");
		 reorientCurv.setEnabled(false);
		 distMapShow.setEnabled(true);
		 loadBSplines.setEnabled(false);
		 saveBSplines.setEnabled(false);
		 addPoints.setEnabled(false);
		 delPoints.setEnabled(false);
		 minDistLab.setEnabled(false);
		 maxDistLab.setEnabled(false);
		 setMinDist.setEnabled(false);
		 setMaxDist.setEnabled(false);
		 panel.removeMouseListener((java.awt.event.MouseListener) panel);
		remove(panel);
		panel = new LSPanel((BSPanel) panel);
		add(panel, BorderLayout.CENTER);
		panel.repaint();
		modelChoice.setText("change to BSplines Model");
	    }
	    else {
		System.out.println("changing to B-Splines");
		cutSplines.setEnabled(true);
		checkOverlap.setEnabled(true);
		regulPoints.setText("Regul the points");
		reorientCurv.setEnabled(true);
		distMapShow.setEnabled(false);
		loadBSplines.setEnabled(true);
		saveBSplines.setEnabled(true);
		 addPoints.setEnabled(true);
		 delPoints.setEnabled(true);
		 minDistLab.setEnabled(true);
		 maxDistLab.setEnabled(true);
		 setMinDist.setEnabled(true);
		 setMaxDist.setEnabled(true);
		 panel.removeMouseListener((java.awt.event.MouseListener) panel);
		remove(panel);
		panel = new BSPanel((LSPanel) panel);
		add(panel, BorderLayout.CENTER);
		panel.repaint();
		modelChoice.setText("change to LevelSet Model");
	    }
	    lr = new LoopRun(panel);
	    loopThread = new Thread(lr);
	    modelSplines = !modelSplines;
	}
	else if(o == loadImage) {
	    if(!isAnApplet) {
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new ImageFilter());
		int returnVal = chooser.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fileSelected = chooser.getSelectedFile();
		    System.out.println("You choosed to open this image file: " + fileSelected);
		    panel.changeImage(fileSelected);
		}
	    }
	    
	    else {
		String s = (String) JOptionPane.showInputDialog(this, "Give the whole URL of your image : ", "Image loading",JOptionPane.QUESTION_MESSAGE, null, null, null);
		if(s != null) {
		    try {
			URL imageUrl = new URL(s);
			if(imageUrl != null)
			    panel.changeImage(imageUrl);
			else
			    JOptionPane.showMessageDialog(this, "Unexistant URL", "Error", JOptionPane.ERROR_MESSAGE);
		    }
		    catch(java.net.MalformedURLException er) {
			JOptionPane.showMessageDialog(this, "Malformed URL", "Error", JOptionPane.ERROR_MESSAGE);
		    }
		}
	    }
	}
	else if(o == saveImage) {
	    if(!isAnApplet) {
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new ImageFilter());
		int returnVal = chooser.showSaveDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fileSelected = chooser.getSelectedFile();
		    System.out.println("You choosed to save current image in file: " + fileSelected);
		    panel.saveImage(fileSelected);
		}
	    }
	}
	else if(o == loadBSplines) {
	    if(!isAnApplet) {
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new BSplinesFilter());
		int returnVal = chooser.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fileSelected = chooser.getSelectedFile();
		    System.out.println("You choosed to open this BSplines file: " + fileSelected);
		    try {
			BufferedReader br = new BufferedReader(new FileReader(fileSelected));
			panel.loadBSplines(br);
		    }
		    catch(java.io.IOException ee) {}
		}
	    }
	    else {
		String s = (String) JOptionPane.showInputDialog(this, "Give the whole URL of your BSplines file : ", "BSplines loading",JOptionPane.QUESTION_MESSAGE, null, null, null);
		try {
		    URL bsUrl = new URL(s);
		    if(bsUrl != null) {
			try { 
			    BufferedReader br = new BufferedReader(new InputStreamReader(bsUrl.openStream()));
			    panel.loadBSplines(br);
			}
			catch(java.io.IOException ee) {}
		    }
		    else
			JOptionPane.showMessageDialog(this, "Unexistant URL", "Error", JOptionPane.ERROR_MESSAGE);
		}
		catch(java.net.MalformedURLException er) {
		    JOptionPane.showMessageDialog(this, "Malformed URL", "Error", JOptionPane.ERROR_MESSAGE);
		}
	    }
	}
	else if(o == saveBSplines) {
	    if(!isAnApplet) {
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new BSplinesFilter());
		int returnVal = chooser.showSaveDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fileSelected = chooser.getSelectedFile();
		    System.out.println("You choosed to save BSplines in this file: " + fileSelected);
		    panel.saveBSplines(fileSelected);
		}
	    }
	    else {
		JFrame jf = new JFrame("Save this in a .bsp file");
		JTextArea jta = new JTextArea(panel.saveBSplinesAsString());
		jta.setFont(new Font("Serif", Font.ITALIC, 14));
		jta.setLineWrap(true);
		jta.setWrapStyleWord(true);
		JScrollPane areaScrollPane = new JScrollPane(jta);
		areaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		areaScrollPane.setPreferredSize(new Dimension(500, 250));
		jf.getContentPane().add(areaScrollPane);
		jf.pack();
		jf.setVisible(true);
	    }
	}
	else if(o == loadParameters) {
	    if(!isAnApplet) {
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new ParametersFilter());
		int returnVal = chooser.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fileSelected = chooser.getSelectedFile();
		    System.out.println("You choosed to open this parameters file: " + fileSelected);
		    try {
			BufferedReader br = new BufferedReader(new FileReader(fileSelected));
			BSparam bspar = panel.loadParameters(br);
			if(bspar != null) {
			    setMinDist.setValue(new Integer(bspar.getClose()));
			    setMaxDist.setValue(new Integer(bspar.getDistant()));
			    switch(bspar.getCriterion()) {
			    case BSPanel.MEAN: meanCritButton.setSelected(true); break;
			    case BSPanel.VARIANCE1: varCritButton1.setSelected(true); break;
			    case BSPanel.VARIANCE2: varCritButton2.setSelected(true); break;
			    case BSPanel.GRADIENT1: gradCritButton1.setSelected(true); break;
			    case BSPanel.GRADIENT2: gradCritButton2.setSelected(true); break;
			    case BSPanel.GRADIENT3: gradCritButton3.setSelected(true); break;
			    case BSPanel.GRADIENT4: gradCritButton4.setSelected(true); break;
			    default: break;
			    }
			    jspin1.setValue(new Double(bspar.getLambda1()));
			    jspin2.setValue(new Double(bspar.getLambda2()));
			    jspin3.setValue(new Double(bspar.getLambda3()));
			}
		    }
		    catch(java.io.IOException ee) {}
		}
	    }
	    else {
		String s = (String) JOptionPane.showInputDialog(this, "Give the whole URL of your parameters file : ", "Parameters loading",JOptionPane.QUESTION_MESSAGE, null, null, null);
		if(s != null) {
		    try {
			URL parUrl = new URL(s);
			if(parUrl != null) {
			     try { 
				 BufferedReader br = new BufferedReader(new InputStreamReader(parUrl.openStream()));
				 BSparam bspar = panel.loadParameters(br);
				 if(bspar != null) {
				     setMinDist.setValue(new Integer(bspar.getClose()));
				     setMaxDist.setValue(new Integer(bspar.getDistant()));
				     switch(bspar.getCriterion()) {
				     case BSPanel.MEAN: meanCritButton.setSelected(true); break;
				     case BSPanel.VARIANCE1: varCritButton1.setSelected(true); break;
				     case BSPanel.VARIANCE2: varCritButton2.setSelected(true); break;
				     case BSPanel.GRADIENT1: gradCritButton1.setSelected(true); break;
				     case BSPanel.GRADIENT2: gradCritButton2.setSelected(true); break;
				     case BSPanel.GRADIENT3: gradCritButton3.setSelected(true); break;
				     case BSPanel.GRADIENT4: gradCritButton4.setSelected(true); break;
				     default: break;
				     }
				     jspin1.setValue(new Double(bspar.getLambda1()));
				     jspin2.setValue(new Double(bspar.getLambda2()));
				     jspin3.setValue(new Double(bspar.getLambda3()));
				 }
			     }
			     catch(java.io.IOException ee) {} 
			}
			else
			    JOptionPane.showMessageDialog(this, "Unexistant URL", "Error", JOptionPane.ERROR_MESSAGE);
		    }
		    catch(java.net.MalformedURLException er) {
			JOptionPane.showMessageDialog(this, "Malformed URL", "Error", JOptionPane.ERROR_MESSAGE);
		    }
		}
	    }
	}
	else if(o == saveParameters) {
	    if(!isAnApplet) {
		JFileChooser chooser = new JFileChooser();
		chooser.addChoosableFileFilter(new ParametersFilter());
		int returnVal = chooser.showSaveDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    File fileSelected = chooser.getSelectedFile();
		    System.out.println("You choosed to save parameters in this file: " + fileSelected);
		    panel.saveParameters(fileSelected);
		}
	    }
	    else {
		JFrame jf = new JFrame("Save this in a .par file");
		JTextArea jta = new JTextArea();
		panel.saveParameters(jta);
		jta.setFont(new Font("Serif", Font.ITALIC, 16));
		jta.setLineWrap(true);
		jta.setWrapStyleWord(true);
		JScrollPane areaScrollPane = new JScrollPane(jta);
		areaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		areaScrollPane.setPreferredSize(new Dimension(250, 150));
		jf.getContentPane().add(areaScrollPane);
		jf.pack();
		jf.setVisible(true);
	    }
	}
	else if(o == meanCritButton) {
	    panel.setCriterion(BSPanel.MEAN);
	}
	else if(o == varCritButton1) {
	    panel.setCriterion(BSPanel.VARIANCE1);
	}
	else if(o == varCritButton2) {
	    panel.setCriterion(BSPanel.VARIANCE2);
	}
	else if(o == gradCritButton1) {
	    panel.setCriterion(BSPanel.GRADIENT1);
	}
	else if(o == gradCritButton2) {
	    panel.setCriterion(BSPanel.GRADIENT2);
	}
	else if(o == gradCritButton3) {
	    panel.setCriterion(BSPanel.GRADIENT3);
	}
	else if(o == gradCritButton4) {
	    panel.setCriterion(BSPanel.GRADIENT4);
	}
	else if(o == cutSplines) {
	    System.out.println("Cut Splines button pressed");
	    panel.cutBSplines();
	    panel.repaint();
	}
	else if(o == checkOverlap) {
	    System.out.println("Check Overlapping button pressed");
	    panel.checkOverlapping();
	    panel.repaint();
	}
	else if(o == applyCrit) {
	    System.out.println("Apply Criterion button pressed");
	    panel.applyCrit();
	    panel.repaint();
	}
	else if(o == regulPoints) {
	    System.out.println("Regul BSPoints button pressed");
	    panel.regulDistances(0);
	    panel.repaint();
	}
	else if(o == reorientCurv){
	    System.out.println("Reorient Curve button pressed");
	    panel.reorientCurv();
	    panel.repaint();
	}
	else if(o == loopEvol){
	    inLoop = !inLoop;
	    if(!inLoop) {
		lr.setInLoop(false);
		loopEvol.setText("Run the loop");
		enabledAllButtonsExceptLoop(true);

		repaint();
		try{
		    loopThread.join();
		}
		catch(InterruptedException ie) {
		    System.out.println("Thread already interrupted");
		}
		loopThread = new Thread(lr);
	    }
	    else {
		lr.setInLoop(true);
		loopEvol.setText("STOP");
		enabledAllButtonsExceptLoop(false);

		repaint();
		loopThread.start();
	    }
	}
	else if( o == helpButton ) {
	    JFrame jf = new JFrame("Help Window");
	    JEditorPane editorPane = new JEditorPane();
	    editorPane.setEditable(false);
	    try {
		URL helpURL = new URL("http://www.i3s.unice.fr/~lingrand/HelpFile.html");
		editorPane.setPage(helpURL);
	    } catch (Exception e2) {
		System.err.println("Couldn't create help URL ");
	    }
	    JScrollPane editorScrollPane = new JScrollPane(editorPane);
	    editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	    editorScrollPane.setPreferredSize(new Dimension(450, 300));
	    jf.getContentPane().add(editorScrollPane);
	    jf.pack();
	    jf.setVisible(true);
	}
	else if( o == addPoints ) {
	    panel.regulDistances(1);
	    panel.repaint();
	}
	else if( o == delPoints ) {
	    panel.regulDistances(-1);
	    panel.repaint();
	}
	else if( o == captureMode ) {
	    if(panel.getCapture()) {
		panel.setCapture(false);
		captureMode.setText("enable capture");
	    }
	    else {
		panel.setCapture(true);
		captureMode.setText("disable capture");
	    }
	}
	else if( o == smoothingMode) {
	    if(panel.getSmoothing()) {
		panel.setSmoothing(false);
		smoothingMode.setText("enable smoothing");
	    }
	    else {
		panel.setSmoothing(true);
		smoothingMode.setText("disable smoothing");
	    }
	}
	else {
	    System.out.println("Don't know. "+ command + " was pressed.");
	}
    }

    /**
     * during the "loop evolution" it is not allowed to change any parameter
     **/
    private void enabledAllButtonsExceptLoop(boolean b) {
	cutSplines.setEnabled(b);
	checkOverlap.setEnabled(b);
	applyCrit.setEnabled(b);
	regulPoints.setEnabled(b);
	reorientCurv.setEnabled(b);
	jspin1.setEnabled(b);
	jspin2.setEnabled(b);
	jspin3.setEnabled(b);
	meanCritButton.setEnabled(b);
	varCritButton1.setEnabled(b);
	varCritButton2.setEnabled(b);
	gradCritButton1.setEnabled(b);
	gradCritButton2.setEnabled(b);
	gradCritButton3.setEnabled(b);
	gradCritButton4.setEnabled(b);
	clearButton.setEnabled(b);
	distMapShow.setEnabled(b);
	modelChoice.setEnabled(b);
	loadImage.setEnabled(b);
	saveImage.setEnabled(b);
	loadBSplines.setEnabled(b);
	saveBSplines.setEnabled(b);
	loadParameters.setEnabled(b);
	saveParameters.setEnabled(b);
	addPoints.setEnabled(b);
	delPoints.setEnabled(b);
	minDistLab.setEnabled(b);
	maxDistLab.setEnabled(b);
	setMinDist.setEnabled(b);
	setMaxDist.setEnabled(b);
	captureMode.setEnabled(b);
	smoothingMode.setEnabled(b);
	
	if(b) { // color for enabled
	    pmc.setForeground(FUSCHIA);
	    pmc2.setForeground(FUSCHIA);
	    pmc3.setForeground(FUSCHIA);
	    prl.setForeground(FUSCHIA);
	    pll.setForeground(FUSCHIA);
	    pel.setForeground(FUSCHIA);
	    ppl.setForeground(FUSCHIA);
	    pfl.setForeground(FUSCHIA);
	}
	else { // color for disabled
	    pmc.setForeground(Color.PINK);
	    pmc2.setForeground(Color.PINK);
	    pmc3.setForeground(Color.PINK);
	    prl.setForeground(Color.PINK);
	    pll.setForeground(Color.PINK);
	    pel.setForeground(Color.PINK);
	    ppl.setForeground(Color.PINK);
	    pfl.setForeground(Color.PINK);
	}
    }
}

/**
 * LoopRun implements the interface Runnable for creating a thread for the loop in 
 * the algorithm.
 **/

class LoopRun implements Runnable {

    private SegPanel panel;
    private boolean inLoop = true;

    public LoopRun(SegPanel p) {
	panel = p;
    }

    public void run() {
	while(inLoop) { panel.evolve(true); panel.evolve(false);}
    }

    public void setInLoop(boolean b) {
	inLoop = b;
    }
}

/**
 * ImageFilter is a @link{FileFilter} for images in JPEG, GIF and PNG format
 **/
class ImageFilter extends FileFilter {

      public boolean accept(File f) {
        if (f.isDirectory())
            return true;

	String s = f.getName();
        int i = s.lastIndexOf('.');
	if (i > 0 &&  i < s.length() - 1) {
	    String extension = s.substring(i+1).toLowerCase();
            if (extension.equals("jpeg") ||
                extension.equals("jpg") ||
                extension.equals("gif") ||
                extension.equals("png"))
		return true;
	}
        return false;
    }

    //The description of this filter
    public String getDescription() {
        return "Image files";
    }
}

/** 
 * BSPlinesFilter is a @link{FileFilter} for BSplines files, assuming that the extension is .bsp
 **/
class BSplinesFilter extends FileFilter {

      public boolean accept(File f) {
        if (f.isDirectory())
            return true;

	String s = f.getName();
        int i = s.lastIndexOf('.');
	if (i > 0 &&  i < s.length() - 1)
	    if(s.substring(i+1).toLowerCase().equals("bsp"))
		return true;

        return false;
    }

    //The description of this filter
    public String getDescription() {
        return "BSplines files";
    }
}
/** 
 * ParametersFilter is a @link{FileFilter} for parameters files, assuming that the extension is .par
 **/
class ParametersFilter extends FileFilter {

      public boolean accept(File f) {
        if (f.isDirectory())
            return true;

	String s = f.getName();
        int i = s.lastIndexOf('.');
	if (i > 0 &&  i < s.length() - 1)
	    if(s.substring(i+1).toLowerCase().equals("par"))
		return true;

        return false;
    }

    //The description of this filter
    public String getDescription() {
        return "BSplines evolution algorithms parameters files";
    }
}
